import random
import time
import matplotlib.pyplot as plt
from statistics import mean

def tri_insertion_iterative(L):
	K = []
	while L:
		m = min(L)
		L.remove(m)
		K.append(m)
	return K


def tri_fusion(L):
	n = len(L)
	if n <= 1:
		return L
	K1 = tri_fusion(L[:n//2])
	K2 = tri_fusion(L[n//2:])
	# On fusionne
	resultat = []
	while bool(K1) and bool(K2):
		# Tant que les deux listes sont non vides
		if K1[0] > K2[0]:
			resultat.append(K2.pop(0))
		else:
			resultat.append(K1.pop(0))
	if K2:
		resultat.extend(K2)
	if K1:
		resultat.extend(K1)
	return resultat

def tri_python(L):
    return sorted(L)


if __name__ == "__main__":
	fcts = [tri_insertion_iterative, tri_fusion, tri_python]
	plt.xlabel("Taille de la liste")
	plt.title("Temps d'exécution des fonctions de tri")

	points = 70
	degree = 2
	repetition = 1
	fig, axs = plt.subplots(1, len(fcts), constrained_layout=True)

	for i, f in enumerate(fcts):
		random.seed(42) # fixe un graîne d'aléa identique, pour avoir toujours les mêmes exemples
		resultats = []
		ax = axs[i]
		for i in range(1, points):
			i = i**degree
			L = random.choices(list(range(i)), k=i)
			temps_calculs = []
			resultats.append(temps_calculs)
			for _ in range(repetition):
				init_time = time.time_ns()
				_ = f(L)
				stop_time = time.time_ns()
				temps_calculs.append(stop_time - init_time)

		ax.plot(
			[(i**degree) for i in range(1, points)],
			[mean(x)/10**3 for x in  resultats],
			marker=".",
			label=f.__name__
			)
		ax.set_ylabel("Temps (en µs)")
		ax.set_xlabel("Taille de la liste")
		ax.legend()
	plt.savefig("tri_total.png")
